@@ -81,6 +81,7 @@
 #define MAX_MESSAGES				17
 #define MISS_COUNT_CONST			5
 #define BLOCK_UNLISTED_IPS			1
+#define CANCEL_TOKEN_HOLD_ON_RETRANSMIT		0
 /* This constant is not used for knet */
 #define UDP_NETMTU                              1500
 
@@ -144,6 +145,8 @@ static void *totem_get_param_by_name(struct totem_config *totem_config, const ch
 		return totem_config->knet_compression_model;
 	if (strcmp(param_name, "totem.block_unlisted_ips") == 0)
 		return &totem_config->block_unlisted_ips;
+	if (strcmp(param_name, "totem.cancel_token_hold_on_retransmit") == 0)
+		return &totem_config->cancel_token_hold_on_retransmit;
 
 	return NULL;
 }
@@ -365,6 +368,9 @@ void totem_volatile_config_read (struct totem_config *totem_config, icmap_map_t
 
 	totem_volatile_config_set_boolean_value(totem_config, temp_map, "totem.block_unlisted_ips", deleted_key,
 	    BLOCK_UNLISTED_IPS);
+
+	totem_volatile_config_set_boolean_value(totem_config, temp_map, "totem.cancel_token_hold_on_retransmit",
+	    deleted_key, CANCEL_TOKEN_HOLD_ON_RETRANSMIT);
 }
 
 int totem_volatile_config_validate (
@@ -535,19 +541,16 @@ static int totem_get_crypto(struct totem_config *totem_config, icmap_map_t map,
 	const char *tmp_cipher;
 	const char *tmp_hash;
 	const char *tmp_model;
+	char *crypto_model_str;
+	int res = 0;
 
 	tmp_hash = "none";
 	tmp_cipher = "none";
 	tmp_model = "none";
 
-	if (icmap_get_string_r(map, "totem.crypto_model", &str) == CS_OK) {
-		if (strcmp(str, "nss") == 0) {
-			tmp_model = "nss";
-		}
-		if (strcmp(str, "openssl") == 0) {
-			tmp_model = "openssl";
-		}
-		free(str);
+	crypto_model_str = NULL;
+	if (icmap_get_string_r(map, "totem.crypto_model", &crypto_model_str) == CS_OK) {
+		tmp_model = crypto_model_str;
 	} else {
 		tmp_model = "nss";
 	}
@@ -601,12 +604,19 @@ static int totem_get_crypto(struct totem_config *totem_config, icmap_map_t map,
 	if ((strcmp(tmp_cipher, "none") != 0) &&
 	    (strcmp(tmp_hash, "none") == 0)) {
 		*error_string = "crypto_cipher requires crypto_hash with value other than none";
-		return -1;
+		res = -1;
+
+		goto out_free_crypto_model_str;
 	}
 
 	if (strcmp(tmp_model, "none") == 0) {
-		*error_string = "crypto_model should be 'nss' or 'openssl'";
-		return -1;
+		/*
+		 * Shouldn't happen because it is handled by coroparse
+		 */
+		*error_string = "invalid crypto_model";
+		res = -1;
+
+		goto out_free_crypto_model_str;
 	}
 
 	if (strcmp(tmp_cipher, totem_config->crypto_cipher_type) ||
@@ -615,11 +625,19 @@ static int totem_get_crypto(struct totem_config *totem_config, icmap_map_t map,
 		totem_config->crypto_changed = 1;
 	    }
 
-	strncpy(totem_config->crypto_cipher_type, tmp_cipher, CONFIG_STRING_LEN_MAX);
-	strncpy(totem_config->crypto_hash_type, tmp_hash, CONFIG_STRING_LEN_MAX);
-	strncpy(totem_config->crypto_model, tmp_model, CONFIG_STRING_LEN_MAX);
+	strncpy(totem_config->crypto_cipher_type, tmp_cipher, CONFIG_STRING_LEN_MAX - 1);
+	totem_config->crypto_cipher_type[CONFIG_STRING_LEN_MAX - 1] = '\0';
 
-	return 0;
+	strncpy(totem_config->crypto_hash_type, tmp_hash, CONFIG_STRING_LEN_MAX - 1);
+	totem_config->crypto_hash_type[CONFIG_STRING_LEN_MAX - 1] = '\0';
+
+	strncpy(totem_config->crypto_model, tmp_model, CONFIG_STRING_LEN_MAX - 1);
+	totem_config->crypto_model[CONFIG_STRING_LEN_MAX - 1] = '\0';
+
+out_free_crypto_model_str:
+	free(crypto_model_str);
+
+	return (res);
 }
 
 static int nodelist_byname(icmap_map_t map, const char *find_name, int strip_domain)
@@ -1404,10 +1422,37 @@ static int put_nodelist_members_to_config(struct totem_config *totem_config, icm
 
 					log_printf(LOGSYS_LEVEL_DEBUG,
 						   "Generated nodeid = " CS_PRI_NODE_ID " for %s", nodeid, str);
+
 					free(str);
+					/*
+					 * Put nodeid back to nodelist to make cfgtool work
+					 */
+					snprintf(tmp_key, ICMAP_KEYNAME_MAXLEN, "nodelist.node.%u.nodeid", node_pos);
+					/*
+					 * Not critical
+					 */
+					(void)icmap_set_uint32_r(map, tmp_key, nodeid);
 				}
 			}
 
+			if (!nodeid && totem_config->transport_number == TOTEM_TRANSPORT_KNET) {
+				sprintf(error_string_response,
+				    "Knet requires an explicit nodeid to be specified "
+				    "for address '%s'.", node_addr_str);
+				*error_string = error_string_response;
+
+				return (-1);
+			}
+
+			if (totem_config->transport_number == TOTEM_TRANSPORT_KNET && nodeid >= KNET_MAX_HOST) {
+				sprintf(error_string_response,
+				    "Knet requires nodeid to be less than %u "
+				    "for address '%s'.", KNET_MAX_HOST, node_addr_str);
+				*error_string = error_string_response;
+
+				return (-1);
+			}
+
 			member_count = totem_config->interfaces[linknumber].member_count;
 			res = totemip_parse(&totem_config->interfaces[linknumber].member_list[member_count],
 						node_addr_str, totem_config->ip_version);
@@ -1743,7 +1788,7 @@ extern int totem_config_read (
 	uint16_t u16;
 	int i;
 	int local_node_pos;
-	int nodeid_set;
+	uint32_t u32;
 
 	*warnings = 0;
 
@@ -1799,7 +1844,9 @@ extern int totem_config_read (
 		free(str);
 	}
 
-	icmap_get_uint32("totem.nodeid", &totem_config->node_id);
+	if (icmap_get_uint32("totem.nodeid", &u32) == CS_OK) {
+		*warnings |= TOTEM_CONFIG_WARNING_TOTEM_NODEID_SET;
+	}
 
 	totem_config->clear_node_high_bit = 0;
 	if (icmap_get_string("totem.clear_node_high_bit", &str) == CS_OK) {
@@ -1892,12 +1939,12 @@ extern int totem_config_read (
 		local_node_pos = find_local_node(icmap_get_global_map(), 1);
 		if (local_node_pos != -1) {
 
+			assert(totem_config->node_id == 0);
+
 			snprintf(tmp_key, ICMAP_KEYNAME_MAXLEN, "nodelist.node.%u.nodeid", local_node_pos);
+			(void)icmap_get_uint32(tmp_key, &totem_config->node_id);
+
 
-			nodeid_set = (totem_config->node_id != 0);
-			if (icmap_get_uint32(tmp_key, &totem_config->node_id) == CS_OK && nodeid_set) {
-				*warnings |= TOTEM_CONFIG_WARNING_TOTEM_NODEID_IGNORED;
-			}
 			if ((totem_config->transport_number == TOTEM_TRANSPORT_KNET) && (!totem_config->node_id)) {
 				*error_string = "Knet requires an explicit nodeid for the local node";
 				return -1;
