@@ -44,8 +44,8 @@
 
 #define TOTEM_CONFIG_WARNING_MEMBERS_IGNORED		(1<<1)
 #define TOTEM_CONFIG_WARNING_MEMBERS_DEPRECATED		(1<<2)
-#define TOTEM_CONFIG_WARNING_TOTEM_NODEID_IGNORED	(1<<3)
-#define TOTEM_CONFIG_BINDNETADDR_NODELIST_SET	(1<<4)
+#define TOTEM_CONFIG_WARNING_TOTEM_NODEID_SET		(1<<3)
+#define TOTEM_CONFIG_BINDNETADDR_NODELIST_SET		(1<<4)
 
 extern int totem_config_read (
 	struct totem_config *totem_config,
