@@ -44,7 +44,7 @@
 #include <corosync/corotypes.h>
 #include <corosync/corodefs.h>
 #include <corosync/coroapi.h>
-#include <corosync/list.h>
+#include <qb/qblist.h>
 #include <corosync/logsys.h>
 #include <corosync/icmap.h>
 #include "fsm.h"
@@ -89,7 +89,7 @@ static uint64_t tickle_timeout = (WD_DEFAULT_TIMEOUT_MS / 2);
 static int dog = -1;
 static corosync_timer_handle_t wd_timer;
 static int watchdog_ok = 1;
-static char *watchdog_device = "/dev/watchdog";
+static char *watchdog_device = NULL;
 
 struct corosync_service_engine wd_service_engine = {
 	.name			= "corosync watchdog service",
@@ -109,7 +109,7 @@ struct corosync_service_engine wd_service_engine = {
 	.exec_dump_fn		= NULL
 };
 
-static DECLARE_LIST_INIT (confchg_notify);
+static QB_LIST_DECLARE (confchg_notify);
 
 /*
  * F S M
@@ -221,15 +221,15 @@ static int32_t wd_resource_state_is_ok (struct resource *ref)
 	uint64_t allowed_period;
 	char key_name[ICMAP_KEYNAME_MAXLEN];
 
-	snprintf(key_name, ICMAP_KEYNAME_MAXLEN, "%s%s", ref->res_path, "last_updated");
-	if (icmap_get_uint64(key_name, &last_updated) != CS_OK) {
+	if ((snprintf(key_name, ICMAP_KEYNAME_MAXLEN, "%s%s", ref->res_path, "last_updated") >= ICMAP_KEYNAME_MAXLEN) ||
+		(icmap_get_uint64(key_name, &last_updated) != CS_OK)) {
 		/* key does not exist.
 		*/
 		return CS_FALSE;
 	}
 
-	snprintf(key_name, ICMAP_KEYNAME_MAXLEN, "%s%s", ref->res_path, "state");
-	if (icmap_get_string(key_name, &state) != CS_OK || strcmp(state, "disabled") == 0) {
+	if ((snprintf(key_name, ICMAP_KEYNAME_MAXLEN, "%s%s", ref->res_path, "state") >= ICMAP_KEYNAME_MAXLEN) ||
+		(icmap_get_string(key_name, &state) != CS_OK || strcmp(state, "disabled") == 0)) {
 		/* key does not exist.
 		*/
 		if (state != NULL)
@@ -279,8 +279,8 @@ static void wd_config_changed (struct cs_fsm* fsm, int32_t event, void * data)
 
 	next_timeout = ref->check_timeout;
 
-	snprintf(key_name, ICMAP_KEYNAME_MAXLEN, "%s%s", ref->res_path, "poll_period");
-	if (icmap_get_uint64(ref->res_path, &tmp_value) == CS_OK) {
+	if ((snprintf(key_name, ICMAP_KEYNAME_MAXLEN, "%s%s", ref->res_path, "poll_period") >= ICMAP_KEYNAME_MAXLEN) ||
+		(icmap_get_uint64(ref->res_path, &tmp_value) == CS_OK)) {
 		if (tmp_value >= WD_MIN_TIMEOUT_MS && tmp_value <= WD_MAX_TIMEOUT_MS) {
 			log_printf (LOGSYS_LEVEL_DEBUG,
 				"poll_period changing from:%"PRIu64" to %"PRIu64".",
@@ -299,8 +299,8 @@ static void wd_config_changed (struct cs_fsm* fsm, int32_t event, void * data)
 		}
 	}
 
-	snprintf(key_name, ICMAP_KEYNAME_MAXLEN, "%s%s", ref->res_path, "recovery");
-	if (icmap_get_string(key_name, &ref->recovery) != CS_OK) {
+	if ((snprintf(key_name, ICMAP_KEYNAME_MAXLEN, "%s%s", ref->res_path, "recovery") >= ICMAP_KEYNAME_MAXLEN) ||
+		(icmap_get_string(key_name, &ref->recovery) != CS_OK)) {
 		/* key does not exist.
 		 */
 		log_printf (LOGSYS_LEVEL_WARNING,
@@ -308,8 +308,8 @@ static void wd_config_changed (struct cs_fsm* fsm, int32_t event, void * data)
 		cs_fsm_state_set(&ref->fsm, WD_S_STOPPED, ref, wd_fsm_cb);
 		return;
 	}
-	snprintf(key_name, ICMAP_KEYNAME_MAXLEN, "%s%s", ref->res_path, "state");
-	if (icmap_get_string(key_name, &state) != CS_OK) {
+	if ((snprintf(key_name, ICMAP_KEYNAME_MAXLEN, "%s%s", ref->res_path, "state") >= ICMAP_KEYNAME_MAXLEN) ||
+		(icmap_get_string(key_name, &state) != CS_OK)) {
 		/* key does not exist.
 		*/
 		log_printf (LOGSYS_LEVEL_WARNING,
@@ -635,7 +635,7 @@ static int setup_watchdog(void)
 	ENTER();
 
 	if (icmap_get_string("resources.watchdog_device", &str) == CS_OK) {
-		if (strcmp (str, "off") == 0) {
+		if (str[0] == 0 || strcmp (str, "off") == 0) {
 			log_printf (LOGSYS_LEVEL_WARNING, "Watchdog disabled by configuration");
 			free(str);
 			dog = -1;
@@ -643,6 +643,10 @@ static int setup_watchdog(void)
 		} else {
 			watchdog_device = str;
 		}
+	} else {
+		log_printf (LOGSYS_LEVEL_WARNING, "Watchdog not enabled by configuration");
+		dog = -1;
+		return -1;
 	}
 
 	if (access (watchdog_device, W_OK) != 0) {
